package com.ycwl.basic.service.impl.pc;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ycwl.basic.enums.StatisticEnum;
import com.ycwl.basic.exception.BaseException;
import com.ycwl.basic.mapper.StatisticsMapper;
import com.ycwl.basic.mapper.FaceMapper;
import com.ycwl.basic.model.jwt.JwtInfo;
import com.ycwl.basic.model.mobile.face.FaceRecognizeResp;
import com.ycwl.basic.model.mobile.statistic.req.StatisticsRecordAddReq;
import com.ycwl.basic.model.pc.face.entity.FaceEntity;
import com.ycwl.basic.model.pc.face.req.FaceReqQuery;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.task.resp.SearchFaceRespVo;
import com.ycwl.basic.service.pc.FaceService;
import com.ycwl.basic.service.task.TaskFaceService;
import com.ycwl.basic.service.task.TaskService;
import com.ycwl.basic.storage.StorageFactory;
import com.ycwl.basic.storage.adapters.IStorageAdapter;
import com.ycwl.basic.storage.utils.StorageUtil;
import com.ycwl.basic.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import static com.ycwl.basic.constant.FaceConstant.USER_FACE_DB_NAME;

/**
 * @Author：longbinbin
 * @Date：2024/12/2 16:39
 */
@Slf4j
@Service
public class FaceServiceImpl implements FaceService {
    @Autowired
    private FaceMapper faceMapper;
    @Autowired
    private TaskFaceService faceService;
    @Autowired
    private StatisticsMapper statisticsMapper;

    @Value("${face.score}")
    private float faceScore;
    private float strictScore = 90F;
    @Autowired
    private TaskService taskTaskService;

    @Override
    public ApiResponse<PageInfo<FaceRespVO>> pageQuery(FaceReqQuery faceReqQuery) {
        PageHelper.startPage(faceReqQuery.getPageNum(),faceReqQuery.getPageSize());
        List<FaceRespVO> list = faceMapper.list(faceReqQuery);
        PageInfo<FaceRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<List<FaceRespVO>> list(FaceReqQuery faceReqQuery) {
        return ApiResponse.success(faceMapper.list(faceReqQuery));
    }

    @Override
    public ApiResponse<FaceRespVO> getById(Long id) {
        return ApiResponse.success(faceMapper.getById(id));
    }

    @Override
    public ApiResponse<Integer> add(FaceEntity face) {
        face.setId(SnowFlakeUtil.getLongId());
        int i = faceMapper.add(face);
        if (i == 0) {
            return ApiResponse.fail("添加失败");
        }
        return ApiResponse.success(i);
    }

    @Override
    public ApiResponse<Integer> deleteById(Long id) {
        int i = faceMapper.deleteById(id);
        if (i == 0) {
            return ApiResponse.fail("删除失败");
        }
        return ApiResponse.success(i);
    }

    @Override
    public ApiResponse<Integer> deleteByIds(List<Long> ids) {
        int i = faceMapper.deleteByIds(ids);
        if (i == 0) {
            return ApiResponse.fail("删除失败");
        }
        return ApiResponse.success(i);
    }

    @Override
    public ApiResponse<Integer> update(FaceEntity face) {
        int i = faceMapper.update(face);
        if (i == 0) {
            return ApiResponse.fail("修改失败");
        }
        return ApiResponse.success(i);
    }

    @Override
//    @Transactional(rollbackFor = Exception.class)
    public ApiResponse faceUpload(MultipartFile file, Long scenicId) {
        //获取用户id
        JwtInfo worker = JwtTokenUtil.getWorker();
        Long userId = worker.getUserId();
        log.info("当前登录用户信息：{}",worker);

        //1、上传人脸照片
        IStorageAdapter adapter = StorageFactory.use("faces");
        String filePath = StorageUtil.joinPath("user-faces", DateUtils.format(new Date(),"yyyy-MM-dd"));
        String originalFilename = file.getOriginalFilename();
        String suffix = originalFilename.split("\\.", 2)[1];
        String fileName = UUID.randomUUID().toString() + "." + suffix;
        String faceUrl = adapter.uploadFile(file, filePath, fileName);
        SearchFaceRespVo scenicDbSearchResult = faceService.searchFace(scenicId, faceUrl);
        if (scenicDbSearchResult == null) {
            adapter.deleteFile(filePath, fileName);
            throw new BaseException("人脸照片校验失败，请重新上传");
        }
        float score = scenicDbSearchResult.getScore();
        if (score<faceScore) {
            //校验失败，删除，提示重新上传
            adapter.deleteFile(filePath, fileName);
            throw new BaseException("人脸照片校验失败，请重新上传");
        }
        // 2、通过人脸查找用户库
        SearchFaceRespVo userDbSearchResult = faceService.searchFace(USER_FACE_DB_NAME, faceUrl);
        Long newFaceId = SnowFlakeUtil.getLongId();
        FaceEntity faceEntity = new FaceEntity();
        faceEntity.setScore(scenicDbSearchResult.getScore());
        faceEntity.setMatchResult(scenicDbSearchResult.getSearchResultJson());
        if (userDbSearchResult == null) {
            // 都是null了，那得是新的
            faceService.addFaceSample(USER_FACE_DB_NAME, newFaceId.toString(), faceUrl, newFaceId.toString());
        } else if (userDbSearchResult.getSampleListIds() == null || userDbSearchResult.getSampleListIds().isEmpty()) {
            // 没有匹配到过，也得是新的
            faceService.addFaceSample(USER_FACE_DB_NAME, newFaceId.toString(), faceUrl, newFaceId.toString());
        } else if (userDbSearchResult.getFirstMatchRate() < strictScore) {
            // 有匹配结果，但是不匹配旧的
            faceService.addFaceSample(USER_FACE_DB_NAME, newFaceId.toString(), faceUrl, newFaceId.toString());
        } else {
            // 有匹配结果，且能匹配旧的数据
            Long oldFaceId = userDbSearchResult.getSampleListIds().get(0);
            faceEntity.setId(oldFaceId);
        }
        if (scenicDbSearchResult.getFirstMatchRate() != null) {
            faceEntity.setFirstMatchRate(BigDecimal.valueOf(scenicDbSearchResult.getFirstMatchRate()));
        }
        if (scenicDbSearchResult.getSampleListIds() != null) {
            faceEntity.setMatchSampleIds(scenicDbSearchResult.getSampleListIds().stream().map(String::valueOf).collect(Collectors.joining(",")));
        }
        faceEntity.setCreateAt(new Date());
        faceEntity.setScenicId(scenicId);
        faceEntity.setMemberId(userId);
        faceEntity.setFaceUrl(faceUrl);
        if (faceEntity.getId()==null) {
            //新增人脸
            faceEntity.setId(newFaceId);
            faceMapper.add(faceEntity);
            taskTaskService.autoCreateTaskByFaceId(faceEntity.getId());
        } else {
            //2、更新人脸
            faceMapper.update(faceEntity);
            taskTaskService.autoCreateTaskByFaceId(faceEntity.getId());
        }
        StatisticsRecordAddReq statisticsRecordAddReq = new StatisticsRecordAddReq();
        statisticsRecordAddReq.setMemberId(userId);
        statisticsRecordAddReq.setType(StatisticEnum.UPLOAD_FACE.code);
        statisticsRecordAddReq.setScenicId(scenicId);
        statisticsRecordAddReq.setMorphId(faceEntity.getId());
        statisticsMapper.addStatisticsRecord(statisticsRecordAddReq);
        FaceRecognizeResp resp = new FaceRecognizeResp();
        resp.setUrl(faceUrl);
        resp.setFaceId(faceEntity.getId());
        return ApiResponse.success(resp);
    }

    @Override
    public ApiResponse<FaceRespVO> getFaceByMemberId(Long memberId) {
        // TODO: 修改
        return ApiResponse.success(faceMapper.getByMemberId(memberId, 3928516560393736192L));
    }

}
